@@ -6,6 +6,7 @@ from binascii import b2a_hex
 from datetime import datetime
 from cStringIO import StringIO
 from ConfigParser import ConfigParser
+from itertools import islice
 
 import tg
 from pylons import app_globals as g
@@ -293,51 +294,31 @@ class HgImplementation(M.RepositoryImplementation):
         return tree._id
 
     def commits(self, path=None, rev=None, skip=None, limit=None):
-        if skip is None: skip = 0
+        start = 0
+        stop = None
+        if skip is not None:
+            start = skip
+        if limit is not None:
+            stop = start + limit
         if path:
             opts = {'pats': [path], 'default': 'path'}
         else:
             opts = {}
         try:
             m = cmdutil.match(self._hg, **opts)
-        except Exception, e:
-            log.exception(e)
-            return []
-        revs = []
+        except Exception as e:
+            log.exception('Error walking Hg commits: %s' % e)
+            return
         opts = {'rev': ['%s:0' % rev] if rev is not None else rev}
         try:
             revs_iter = cmdutil.walkchangerevs(self._hg, m, opts, lambda *a: None)
-        except Exception, e:
-            log.exception(e)
-            return []
-        for ctx in revs_iter:
-            if limit and limit == len(revs):
-                return revs
-            if skip == 0:
-                revs.append(ctx.hex())
-            else:
-                skip -= 1
-        return revs
+        except Exception as e:
+            log.exception('Error walking Hg commits: %s' % e)
+            return
+        for ctx in islice(revs_iter, start, stop):
+            yield ctx.hex()
 
     def commits_count(self, path=None, rev=None):
-        if path:
-            opts = {'pats': [path], 'default': 'path'}
-        else:
-            opts = {}
-        try:
-            m = cmdutil.match(self._hg, **opts)
-        except Exception, e:
-            log.exception(e)
-            return 0
-        opts = {'rev': ['%s:0' % rev] if rev is not None else rev}
-        try:
-            revs_iter = cmdutil.walkchangerevs(self._hg, m, opts, lambda *a: None)
-        except Exception, e:
-            log.exception(e)
-            return 0
-        count = 0
-        for ctx in revs_iter:
-            count += 1
-        return count
+        return sum(1 for _ in self.commits(path=path, rev=rev))
 
 Mapper.compile_all()
